@@ -38,9 +38,10 @@ char * credit_text[] = {
   "-DESIGNED BY",
   "SAM 'CRISWELL' HART",
   "",
-  "-LEAD PROGRAMMER",
-  "BILL KENDRICK",
+  "-LEAD PROGRAMMERS",
+  "BILL KENDRICK,",
   "NEW BREED SOFTWARE",
+  "DAVID BRUCE",
   "",
   "-ADDITIONAL CODE",
   "GLEN DITCHFIELD",
@@ -48,13 +49,15 @@ char * credit_text[] = {
   "DONNY VISZNEKI",
   "YVES COMBE",
   "DAVID YODER",
-  "DAVID BRUCE",
+  "TIM HOLY",
+  "KARL OVE HUFTHAMMER",
   "",
   "-LEAD ARTIST",
   "SAM HART",
   "",
   "-ADDITIONAL ART",
   "BILL KENDRICK",
+  "KENDRA SWANSON & LINNEA HOLY",
   "",
   "-SOUND EFFECTS",
   "TBA",
@@ -81,10 +84,14 @@ char * credit_text[] = {
   "ST. CATHERINE ELEM., CINCINNATI, OH",
   "WESTWOOD ELEMENTARY, CINCINNATI, OH",
   "LAURA BRUCE",
+  "ROOSEVELT ELEMENTARY, TAMPA, FL",
+  "KENDRA SWANSON AND LINNEA HOLY",
+  "OLD BONHOMME ELEMENTARY,",
+  "ST. LOUIS, MO",
   "",
   "",
   "-WEBSITE",
-  "TUX4KIDS.COM",
+  "TUX4KIDS.DEBIAN.ALIOTH.ORG",
   "", /* The following blanks cause the screen to scroll to complete blank: */
   "",
   "",
@@ -388,7 +395,6 @@ int credits(void)
       
       
       /* Handle any incoming events: */
-      /* TODO allow player to return to Title with mouse click */
       while (SDL_PollEvent(&event) > 0)
 	{
 	  if (event.type == SDL_QUIT)
@@ -409,6 +415,10 @@ int credits(void)
 		  done = 1;
 		}
 	    }
+	  else if (event.type == SDL_MOUSEBUTTONDOWN)
+	    {
+              done = 1;
+	    }
 	}
 
       
