@@ -57,6 +57,7 @@ static int already_in_list(MC_MathQuestion* list, MC_MathQuestion* ptr);
 static int int_to_bool(int i);
 static int sane_value(int i);
 static int abs_value(int i);
+static int randomly_keep(void);
 
 static void print_list(FILE* fp,MC_MathQuestion* list);
 static void print_node(FILE* fp, MC_MathQuestion* ptr);
@@ -119,6 +120,7 @@ int MC_Initialize(void)
   math_opts->max_questions = DEFAULT_MAX_QUESTIONS;
   math_opts->question_copies = DEFAULT_QUESTION_COPIES;
   math_opts->randomize = DEFAULT_RANDOMIZE;
+  math_opts->fraction_to_keep = DEFAULT_FRACTION_TO_KEEP;
   /* set question formats:  */
   math_opts->format_add_answer_last = DEFAULT_FORMAT_ADD_ANSWER_LAST; 
   math_opts->format_add_answer_first = DEFAULT_FORMAT_ADD_ANSWER_FIRST;
@@ -133,30 +135,34 @@ int MC_Initialize(void)
   math_opts->format_div_answer_first = DEFAULT_FORMAT_DIV_ANSWER_FIRST;
   math_opts->format_div_answer_middle = DEFAULT_FORMAT_DIV_ANSWER_MIDDLE;
 
-  /* set addition options */
+  /* set addition options: */
   math_opts->addition_allowed = DEFAULT_ADDITION_ALLOWED;
   math_opts->min_augend = DEFAULT_MIN_AUGEND;
   math_opts->max_augend = DEFAULT_MAX_AUGEND;
   math_opts->min_addend = DEFAULT_MIN_ADDEND;
   math_opts->max_addend = DEFAULT_MAX_ADDEND;
-  /* set subtraction options */
+  /* set subtraction options: */
   math_opts->subtraction_allowed = DEFAULT_SUBTRACTION_ALLOWED;
   math_opts->min_minuend = DEFAULT_MIN_MINUEND;
   math_opts->max_minuend = DEFAULT_MAX_MINUEND;
   math_opts->min_subtrahend = DEFAULT_MIN_SUBTRAHEND;
   math_opts->max_subtrahend = DEFAULT_MAX_SUBTRAHEND;
-  /* set multiplication options */
+  /* set multiplication options: */
   math_opts->multiplication_allowed = DEFAULT_MULTIPLICATION_ALLOWED;
   math_opts->min_multiplier = DEFAULT_MIN_MULTIPLIER;
   math_opts->max_multiplier = DEFAULT_MAX_MULTIPLIER;
   math_opts->min_multiplicand = DEFAULT_MIN_MULTIPLICAND;
   math_opts->max_multiplicand = DEFAULT_MAX_MULTIPLICAND;
-  /* set division options */
+  /* set division options: */
   math_opts->division_allowed = DEFAULT_DIVISION_ALLOWED;
   math_opts->min_divisor = DEFAULT_MIN_DIVISOR;
   math_opts->max_divisor = DEFAULT_MAX_DIVISOR;
   math_opts->min_quotient = DEFAULT_MIN_QUOTIENT;
   math_opts->max_quotient = DEFAULT_MAX_QUOTIENT;
+  /* set typing practice options: */
+  math_opts->typing_practice_allowed = DEFAULT_TYPING_PRACTICE_ALLOWED;
+  math_opts->min_typing_num = DEFAULT_MIN_TYPING_NUM;
+  math_opts->max_typing_num = DEFAULT_MAX_TYPING_NUM;
 
   /* if no negatives to be used, reset any negatives to 0 */
   if (!math_opts->allow_negatives)
@@ -212,7 +218,9 @@ int MC_StartGame(void)
   
   /* clear out old lists if starting another game: (if not done already) */
   delete_list(question_list);
+  question_list = NULL;
   delete_list(wrong_quests);
+  wrong_quests = NULL;
 
   /* set up new list with pointer to top: */
   question_list = generate_list();
@@ -474,6 +482,7 @@ int MC_NotAnsweredCorrectly(MC_FlashCard* fc)
   questions_pending--;
 
   /* add question to wrong_quests list: */
+  {
   MC_MathQuestion* ptr1;
   MC_MathQuestion* ptr2;
 
@@ -530,7 +539,7 @@ int MC_NotAnsweredCorrectly(MC_FlashCard* fc)
 
   return 1;
 }
-
+}
 
 /* Tells user interface if all questions have been answered correctly! */
 /* Requires that at list contained at least one question to start with */
@@ -878,6 +887,21 @@ void MC_SetRandomize(int opt)
   math_opts->randomize = int_to_bool(opt);
 } 
 
+void MC_SetFractionToKeep(float fract)
+{
+  if (!math_opts)
+  {
+    fprintf(stderr, "\nMC_SetRandomize(): math_opts not valid!\n");
+    return;
+  }
+  /* must be between 0 and 1: */
+  if (fract < 0)
+    fract = 0;
+  if (fract > 1)
+    fract = 1;
+  math_opts->fraction_to_keep = fract;
+}
+
 
 /* Set math operations to be used in game: */
 void MC_SetAddAllowed(int opt)
@@ -924,6 +948,15 @@ void MC_SetDivAllowed(int opt)
 }
 
 
+void MC_SetTypingAllowed(int opt)
+{
+  if (!math_opts)
+  {
+    fprintf(stderr, "\nMC_SetTypingAllowed(): math_opts not valid!\n");
+    return;
+  }
+  math_opts->typing_practice_allowed = int_to_bool(opt);
+}
 
 
 
@@ -1169,6 +1202,27 @@ void MC_SetDivMaxQuotient(int opt)
 }
 
 
+/* Set min and max for division: */
+void MC_SetTypeMin(int opt)
+{
+  if (!math_opts)
+  {
+    fprintf(stderr, "\nMC_SetTypeMin(): math_opts not valid!\n");
+    return;
+  }
+  math_opts->min_typing_num = sane_value(opt);
+}
+
+
+void MC_SetTypeMax(int opt)
+{
+  if (!math_opts)
+  {
+    fprintf(stderr, "\nMC_SetTypeMax(): math_opts not valid!\n");
+    return;
+  }
+  math_opts->max_typing_num = sane_value(opt);
+}
 
 
 /*"Get" type methods to query option parameters */
@@ -1240,6 +1294,16 @@ int MC_CopiesRepeatedWrongs(void)
 }
 
 
+float MC_FractionToKeep(void)
+{
+  if (!math_opts)
+  {
+    fprintf(stderr, "\nMC_FractionToKeep(): math_opts not valid!\n");
+    return MC_MATH_OPTS_INVALID;
+  }
+  return math_opts->fraction_to_keep;
+}
+
 
 
 int MC_FormatAddAnswerLast(void)      /* a + b = ?                                               */
@@ -1442,6 +1506,16 @@ int MC_DivAllowed(void)
 }
 
 
+int MC_TypingAllowed(void)
+{
+  if (!math_opts)
+  {
+    fprintf(stderr, "\nMC_TypeAllowed(): math_opts not valid!\n");
+    return MC_MATH_OPTS_INVALID;
+  }
+  return math_opts->typing_practice_allowed;
+}
+
 
 /* Query min and max for addition: */
 int MC_AddMinAugend(void)               /* the "augend" is the first addend i.e. "a" in "a + b = c" */
@@ -1630,6 +1704,28 @@ int MC_DivMaxQuotient(void)
 }
 
 
+/* Query min and max for typing practice: */
+int MC_TypeMin(void)
+{
+  if (!math_opts)
+  {
+    fprintf(stderr, "\nMC_TypeMin(): math_opts not valid!\n");
+    return MC_MATH_OPTS_INVALID;
+  }
+  return math_opts->min_typing_num;
+}
+
+
+int MC_TypeMax(void)
+{
+  if (!math_opts)
+  {
+    fprintf(stderr, "\nMC_TypeMax(): math_opts not valid!\n");
+    return MC_MATH_OPTS_INVALID;
+  }
+  return math_opts->max_typing_num;
+}
+
 
 /* prints struct to file */
 void MC_PrintMathOptions(FILE* fp, int verbose)
@@ -1811,6 +1907,14 @@ void MC_PrintMathOptions(FILE* fp, int verbose)
   fprintf(fp, "max_divisor = %d\n", math_opts->max_divisor);
   fprintf(fp, "min_quotient = %d\n", math_opts->min_quotient);
   fprintf(fp, "max_quotient = %d\n", math_opts->max_quotient);
+
+  fprintf(fp, "\n# Typing practice:\n");
+  fprintf(fp, "min_typing_num = %d\n",math_opts->min_typing_num);
+  fprintf(fp, "max_typing_num = %d\n",math_opts->max_typing_num);
+
+  #ifdef MC_DEBUG
+  printf("\nLeaving MC_PrintMathOptions()\n");
+  #endif
 }
 
 
@@ -1916,6 +2020,11 @@ void clear_negatives(void)
     math_opts->min_quotient = 0;
   if (math_opts->max_quotient < 0)
     math_opts->max_quotient = 0;
+
+  if (math_opts->min_typing_num < 0)
+    math_opts->min_typing_num = 0;
+  if (math_opts->max_typing_num < 0)
+    math_opts->max_typing_num = 0;
 }
 
 /* using parameters from the mission struct, create linked list of "flashcards" */
@@ -1924,9 +2033,9 @@ void clear_negatives(void)
 /* math operations and question formats                                    */
 MC_MathQuestion* generate_list(void)
 {
-  MC_MathQuestion* top_of_list = 0;
-  MC_MathQuestion* end_of_list = 0;
-  MC_MathQuestion* tmp_ptr = 0;
+  MC_MathQuestion* top_of_list = NULL;
+  MC_MathQuestion* end_of_list = NULL;
+  MC_MathQuestion* tmp_ptr = NULL;
 
   int i, j, k;
   int length = 0;
@@ -1938,9 +2047,15 @@ MC_MathQuestion* generate_list(void)
  
   /* add nodes for each math operation allowed */
 
+  #ifdef MC_DEBUG
+  printf("\ngenerating addition questions\n");
+  #endif
 
   if (math_opts->addition_allowed)
   {
+    #ifdef MC_DEBUG
+    printf("\nAddition problems");
+    #endif
     for (i = math_opts->min_augend; i <= math_opts->max_augend; i++)
     {
       for (j = math_opts->min_addend; j <= math_opts->max_addend; j++)
@@ -1957,8 +2072,10 @@ MC_MathQuestion* generate_list(void)
             /* questions like num1 + num2 = ? */
             if (math_opts->format_add_answer_last)
             {
-              /* make sure max_questions not exceeded */
-              if (length < math_opts->max_questions)
+              /* make sure max_questions not exceeded, */
+              /* also check if question being randomly kept or discarded: */
+              if ((length < math_opts->max_questions)
+                 && randomly_keep())
               {
                 tmp_ptr = create_node(i, j, MC_OPER_ADD, i + j, MC_FORMAT_ANS_LAST);
                 top_of_list = insert_node(top_of_list, end_of_list, tmp_ptr);
@@ -1970,8 +2087,10 @@ MC_MathQuestion* generate_list(void)
             /* questions like num1 + ? = num3 */
             if (math_opts->format_add_answer_middle)
             {
-              /* make sure max_questions not exceeded */
-              if (length < math_opts->max_questions)
+              /* make sure max_questions not exceeded, */
+              /* also check if question being randomly kept or discarded: */
+              if ((length < math_opts->max_questions)
+                 && randomly_keep())
               {
                 tmp_ptr = create_node(i, j, MC_OPER_ADD, i + j, MC_FORMAT_ANS_MIDDLE);
                 top_of_list = insert_node(top_of_list, end_of_list, tmp_ptr);
@@ -1983,8 +2102,11 @@ MC_MathQuestion* generate_list(void)
             /* questions like ? + num2 = num3 */
             if (math_opts->format_add_answer_first)
             {
-              /* make sure max_questions not exceeded */
-              if (length < math_opts->max_questions)
+              /* make sure max_questions not exceeded, */
+              /* also check if question being randomly kept or discarded: */
+              if ((length < math_opts->max_questions)
+                 && randomly_keep())
+
               {
                 tmp_ptr = create_node(i, j, MC_OPER_ADD, i + j, MC_FORMAT_ANS_FIRST);
                 top_of_list = insert_node(top_of_list, end_of_list, tmp_ptr);
@@ -1998,8 +2120,15 @@ MC_MathQuestion* generate_list(void)
     }
   }
 
+  #ifdef MC_DEBUG
+  printf("generating subtraction questions\n");
+  #endif
+
   if (math_opts->subtraction_allowed)
   {
+    #ifdef MC_DEBUG
+    printf("\nSubtraction problems");
+    #endif
     for (i = math_opts->min_minuend; i <= math_opts->max_minuend; i++)
     {
       for (j = math_opts->min_subtrahend; j <= math_opts->max_subtrahend; j++)
@@ -2016,8 +2145,11 @@ MC_MathQuestion* generate_list(void)
             /* questions like num1 - num2 = ? */
             if (math_opts->format_sub_answer_last)
             {
-              /* make sure max_questions not exceeded */
-              if (length < math_opts->max_questions)
+              /* make sure max_questions not exceeded, */
+              /* also check if question being randomly kept or discarded: */
+              if ((length < math_opts->max_questions)
+                 && randomly_keep())
+
               {
                 tmp_ptr = create_node(i, j, MC_OPER_SUB, i - j, MC_FORMAT_ANS_LAST);
                 top_of_list = insert_node(top_of_list, end_of_list, tmp_ptr);
@@ -2029,8 +2161,11 @@ MC_MathQuestion* generate_list(void)
             /* questions like num1 - ? = num3 */
             if (math_opts->format_sub_answer_middle)
             {
-              /* make sure max_questions not exceeded */
-              if (length < math_opts->max_questions)
+              /* make sure max_questions not exceeded, */
+              /* also check if question being randomly kept or discarded: */
+              if ((length < math_opts->max_questions)
+                 && randomly_keep())
+
               {
                 tmp_ptr = create_node(i, j, MC_OPER_SUB, i - j, MC_FORMAT_ANS_MIDDLE);
                 top_of_list = insert_node(top_of_list, end_of_list, tmp_ptr);
@@ -2042,8 +2177,10 @@ MC_MathQuestion* generate_list(void)
             /* questions like ? - num2 = num3 */
             if (math_opts->format_sub_answer_first)
             {
-              /* make sure max_questions not exceeded */
-              if (length < math_opts->max_questions)
+              /* make sure max_questions not exceeded, */
+              /* also check if question being randomly kept or discarded: */
+              if ((length < math_opts->max_questions)
+                 && randomly_keep())
               {
                 tmp_ptr = create_node(i, j, MC_OPER_SUB, i - j, MC_FORMAT_ANS_FIRST);
                 top_of_list = insert_node(top_of_list, end_of_list, tmp_ptr);
@@ -2057,8 +2194,15 @@ MC_MathQuestion* generate_list(void)
     }
   }
 
+  #ifdef MC_DEBUG
+  printf("generating multiplication questions\n");
+  #endif
+
   if (math_opts->multiplication_allowed)
   {
+    #ifdef MC_DEBUG
+    printf("\nMultiplication problems");
+    #endif
     for (i = math_opts->min_multiplier; i <= math_opts->max_multiplier; i++)
     {
       for (j = math_opts->min_multiplicand; j <= math_opts->max_multiplicand; j++)
@@ -2075,8 +2219,10 @@ MC_MathQuestion* generate_list(void)
             /* questions like num1 x num2 = ? */
             if (math_opts->format_mult_answer_last)
             {
-              /* make sure max_questions not exceeded */
-              if (length < math_opts->max_questions)
+              /* make sure max_questions not exceeded, */
+              /* also check if question being randomly kept or discarded: */
+              if ((length < math_opts->max_questions)
+                 && randomly_keep())
               {
                 tmp_ptr = create_node(i, j, MC_OPER_MULT, i * j, MC_FORMAT_ANS_LAST);
                 top_of_list = insert_node(top_of_list, end_of_list, tmp_ptr);
@@ -2090,8 +2236,10 @@ MC_MathQuestion* generate_list(void)
             if ((math_opts->format_mult_answer_middle)
              && (i != 0)) 
             {
-                 /* make sure max_questions not exceeded */
-              if (length < math_opts->max_questions)
+              /* make sure max_questions not exceeded, */
+              /* also check if question being randomly kept or discarded: */
+              if ((length < math_opts->max_questions)
+                 && randomly_keep())
               {
                 tmp_ptr = create_node(i, j, MC_OPER_MULT, i * j, MC_FORMAT_ANS_MIDDLE);
                 top_of_list = insert_node(top_of_list, end_of_list, tmp_ptr);
@@ -2105,8 +2253,10 @@ MC_MathQuestion* generate_list(void)
             if ((math_opts->format_mult_answer_first)
              && (j != 0))
             {
-              /* make sure max_questions not exceeded */
-              if (length < math_opts->max_questions)
+              /* make sure max_questions not exceeded, */
+              /* also check if question being randomly kept or discarded: */
+              if ((length < math_opts->max_questions)
+                 && randomly_keep())
               {
                 tmp_ptr = create_node(i, j, MC_OPER_MULT, i * j, MC_FORMAT_ANS_FIRST);
                 top_of_list = insert_node(top_of_list, end_of_list, tmp_ptr);
@@ -2120,8 +2270,15 @@ MC_MathQuestion* generate_list(void)
     }
   }
 
+  #ifdef MC_DEBUG
+  printf("generating division questions\n");
+  #endif
+
   if (math_opts->division_allowed)
   {
+    #ifdef MC_DEBUG
+    printf("\nDivision problems");
+    #endif
     for (i = math_opts->min_quotient; i <= math_opts->max_quotient; i++)
     {
       for (j = math_opts->min_divisor; j <= math_opts->max_divisor; j++)
@@ -2140,8 +2297,10 @@ MC_MathQuestion* generate_list(void)
             /* questions like num1 / num2 = ? */
             if (math_opts->format_div_answer_last)
             {
-              /* make sure max_questions not exceeded */
-              if (length < math_opts->max_questions)
+              /* make sure max_questions not exceeded, */
+              /* also check if question being randomly kept or discarded: */
+              if ((length < math_opts->max_questions)
+                 && randomly_keep())
               {
                 tmp_ptr = create_node(i * j, j, MC_OPER_DIV, i, MC_FORMAT_ANS_LAST);
                 top_of_list = insert_node(top_of_list, end_of_list, tmp_ptr);
@@ -2154,8 +2313,10 @@ MC_MathQuestion* generate_list(void)
             if ((math_opts->format_div_answer_middle)
                && (i))      /* This avoids creating indeterminate questions: 0/? = 0 */
             {
-              /* make sure max_questions not exceeded */
-              if (length < math_opts->max_questions)
+              /* make sure max_questions not exceeded, */
+              /* also check if question being randomly kept or discarded: */
+              if ((length < math_opts->max_questions)
+                 && randomly_keep())
               {
                 tmp_ptr = create_node(i * j, j, MC_OPER_DIV, i, MC_FORMAT_ANS_MIDDLE);
                 top_of_list = insert_node(top_of_list, end_of_list, tmp_ptr);
@@ -2167,8 +2328,10 @@ MC_MathQuestion* generate_list(void)
             /* questions like ? / num2  = num3 */
             if (math_opts->format_div_answer_first)
             {
-              /* make sure max_questions not exceeded */
-              if (length < math_opts->max_questions)
+              /* make sure max_questions not exceeded, */
+              /* also check if question being randomly kept or discarded: */
+              if ((length < math_opts->max_questions)
+                 && randomly_keep())
               {
                 tmp_ptr = create_node(i * j, j, MC_OPER_DIV, i, MC_FORMAT_ANS_FIRST);
                 top_of_list = insert_node(top_of_list, end_of_list, tmp_ptr);
@@ -2182,6 +2345,38 @@ MC_MathQuestion* generate_list(void)
     }
   }
 
+  #ifdef MC_DEBUG
+  printf("generating typing practice questions\n");
+  #endif
+
+  if (math_opts->typing_practice_allowed)
+  {
+    #ifdef MC_DEBUG
+    printf("\nTyping problems");
+    #endif
+    for (i = math_opts->min_typing_num; i <= math_opts->max_typing_num; i++)
+    {
+      /* check if max_answer exceeded or if question */
+      /* contains undesired negative values:         */
+      if (validate_question(i, i, i))
+      {  
+        /* put in the desired number of copies: */
+        for (k = 0; k < math_opts->question_copies; k++)
+        {
+          /* make sure max_questions not exceeded, */
+          /* also check if question being randomly kept or discarded: */
+          if ((length < math_opts->max_questions)
+               && randomly_keep())
+          {
+            tmp_ptr = create_node(i, i, MC_OPER_TYPING_PRACTICE, i, MC_FORMAT_ANS_LAST);
+            top_of_list = insert_node(top_of_list, end_of_list, tmp_ptr);
+            end_of_list = tmp_ptr;
+            length++; 
+          } 
+        }
+      }
+    }
+  }
   #ifdef MC_DEBUG
   length = list_length(top_of_list); 
   printf("\nlength before randomization:\t%d", length); 
@@ -2229,15 +2424,24 @@ int validate_question(int n1, int n2, int n3)
 /* create a new node and return a pointer to it */
 MC_MathQuestion* create_node(int n1, int n2, int op, int ans, int f)
 {
-  MC_MathQuestion* ptr;
-  ptr = malloc(sizeof(MC_MathQuestion));
+  MC_MathQuestion* ptr = NULL;
+
+  ptr = (MC_MathQuestion*)malloc(sizeof(MC_MathQuestion));
+
+  if (!ptr)
+  {
+    fprintf(stderr, "create_node() - malloc() failed!\n");
+    return NULL;
+  }
+
   ptr->card.num1 = n1;
   ptr->card.num2 = n2;  
   ptr->card.num3 = ans;
   ptr->card.operation = op;
   ptr->card.format = f;
-  ptr->next = 0;
-  ptr->previous =0;
+  ptr->next = NULL;
+  ptr->previous = NULL;
+
 
   /* creating formula_string  and answer_string is a little more work: */
   {
@@ -2265,11 +2469,18 @@ MC_MathQuestion* create_node(int n1, int n2, int op, int ans, int f)
         oper_char = '/';
         break; 
       }
+      case MC_OPER_TYPING_PRACTICE:
+      {
+        snprintf(ptr->card.formula_string, MC_FORMULA_LEN, "%d",ptr->card.num1);
+        snprintf(ptr->card.answer_string, MC_ANSWER_LEN, "%d",ptr->card.num1);
+        return ptr;  /* Don't need to look at formats for this case. */
+      }
       default:
       {
         fprintf(stderr, "\nIn create_node(): invalid math operation\n");
         free(ptr);
         ptr = 0;
+
         return 0;
       }
     }
@@ -2308,6 +2519,8 @@ MC_MathQuestion* create_node(int n1, int n2, int op, int ans, int f)
         fprintf(stderr, "\ncreate_node() - invalid question format\n");
         free(ptr);
         ptr = 0;
+
+ 
         return 0;
       }
     }
@@ -2480,7 +2693,7 @@ void print_list(FILE* fp, MC_MathQuestion* list)
     fprintf(fp, "\nprint_list(): list empty or pointer invalid\n");
     return;
   }
- 
+ {
   MC_MathQuestion* ptr = list;
   while (ptr)
   {
@@ -2488,7 +2701,7 @@ void print_list(FILE* fp, MC_MathQuestion* list)
     ptr = ptr->next;
   }
 }
-
+}
 
 
 /* Don't need this much now that formula_string part of card struct:  */
@@ -2699,3 +2912,24 @@ int abs_value(int i)
   else
     return -i;
 }
+
+
+/* Returns true at probability set by math_opts->fraction_to_keep */
+int randomly_keep(void)
+{
+  int random;
+
+  if (!math_opts)
+    return 0;
+
+  /* Skip random number generation if keeping all (default) */
+  if (1 == math_opts->fraction_to_keep)
+    return 1;
+
+  random = rand() % 1000;
+
+  if (random < (math_opts->fraction_to_keep * 1000))
+    return 1;
+  else
+    return 0;
+}
