@@ -8,11 +8,13 @@
   bill@newbreedsoftware.com
   http://www.newbreedsoftware.com/
 
+  Modified extensively by David Bruce
+  dbruce@tampabay.rr.com
 
   Part of "Tux4Kids" Project
-  http://www.tux4kids.org/
+  http://tux4kids.alioth.debian.org
       
-  August 26, 2001 - January 3, 2005
+  August 26, 2001 - July 11, 2007
 */
 
 
@@ -27,300 +29,21 @@
 #include "options.h"
 #include "fileops.h"
 #include "setup.h"
-#include "playsound.h"
 #include "game.h"
 #include "tuxmath.h"
 
 /* FIXME figure out what oper_override is supposed to do and make sure */
 /* this file behaves accordingly! */
 
-int opers[NUM_OPERS], range_enabled[NUM_Q_RANGES];
-
-
-
-/* Local (to options.c) 'globals': */
-/* moved to file scope to allow update_selected_option() to use them DSB */
-static SDL_Rect dest;
-static char tmp_str[10];
-static unsigned char range_bits;
-static int i, j, x, y;
+//int opers[NUM_OPERS], range_enabled[NUM_Q_RANGES];
 
 /* file scope only now that accessor functions used: */
 static game_option_type* game_options;
 
 /*local function prototypes: */
-static void update_selected_option(int option);
 static int int_to_bool(int i);
-static void print_game_options(FILE* fp, int verbose);
 
 
-/* Main options() function - called from title(): */
-int options(void)
-{
-  int opt, old_opt, done, quit, img, blinking;
-  SDL_Event event;
-  Uint32 last_time, now_time;
-  SDLKey key;
-  
-  
-  /* Clear window: */
-  
-  SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 0, 0, 0));
-
-  dest.x = (screen->w - images[IMG_OPTIONS]->w) / 2;
-  dest.y = 0;
-  SDL_BlitSurface(images[IMG_OPTIONS], NULL, screen, &dest);
-
-  /* Syncrhonize old opers[] array with selections from MathCards */
-  opers[OPT_OP_ADD] = MC_AddAllowed();
-  opers[OPT_OP_SUB] = MC_SubAllowed();
-  opers[OPT_OP_MUL] = MC_MultAllowed();
-  opers[OPT_OP_DIV] = MC_DivAllowed();
-
-  /* FIXME range_enabled[] and oper_override will be going away */
-  for (i = 0; i < NUM_Q_RANGES; i++)
-  { 
-    range_enabled[i] = 1;
-  }
-
-  /* Draw options: */
-
-  for (i = 0; i < NUM_OPTS; i++)
-  {
-    y = (images[IMG_OPTIONS]->h + 8 + (i * images[IMG_TUX_HELMET1]->h));
-    
-    dest.x = 32 + images[IMG_TUX_HELMET1]->w + 4;
-    dest.y = y;
-    
-    SDL_BlitSurface(images[IMG_OPT_ADDITION + i], NULL, screen, &dest);
-
-
-    /* Checkmarks for operators: */
-
-    if (i >= OPT_OP_ADD && i < OPT_OP_ADD + NUM_OPERS)
-    {
-      dest.x = screen->w - images[IMG_OPT_CHECK]->w - 16;
-      dest.y = y;
-      
-      SDL_BlitSurface(images[IMG_OPT_CHECK + opers[i]], NULL, screen, &dest);
-    }
-    else if (i == OPT_A_MAX)
-    {
-      /* Maximum answer: */
-
-      snprintf(tmp_str, sizeof(tmp_str), "%04d", MC_MaxAnswer());
-      draw_nums(tmp_str,
-		screen->w - ((images[IMG_NUMS]->w / 14) * 2) - 16,
-		y + images[IMG_OPT_MAX_ANSWER]->h);
-
-      /* Using "* 2" instead of "* 4" (even though string is 4 digits long)
-         because "draw_nums()" actually centers around x; not left-justifies */
-    }
-    else if (i == OPT_A_SPEED)
-    {
-      /* Speed control: */
-
-      snprintf(tmp_str, sizeof(tmp_str), "%.1f", game_options->speed);
-      draw_nums(tmp_str,
-		screen->w - ((images[IMG_NUMS]->w / 14) * 2) - 16,
-		y + images[IMG_OPT_SPEED]->h);
-
-      /* Using "* 2" instead of "* 4" (even though string is 4 digits long)
-         because "draw_nums()" actually centers around x; not left-justifies */
-    }
-    else if (i == OPT_Q_RANGE)
-    {
-      x = 32 + images[IMG_TUX_HELMET1]->w + 4 + 64;
-      y = y + images[IMG_TUX_HELMET1]->h;
-
-      for (j = 0; j < NUM_Q_RANGES; j++)
-      {
-	dest.x = x;
-	dest.y = y;
-
-	SDL_BlitSurface(images[IMG_OPT_RNG_1_5 + j * 2 + range_enabled[j]],
-			NULL,
-			screen, &dest);
-
-	x = x + images[IMG_OPT_RNG_1_5 + j * 2 + range_enabled[j]]->w + 16;
-      }
-    }
-  }
-
-
-  /* Get bits of what ranges are available: */
-
-  range_bits = 0;
-  for (j = 0; j < NUM_Q_RANGES; j++)
-  {
-    range_bits = range_bits << 1;
-    if (range_enabled[j])
-      range_bits = range_bits | 1;
-  }
-
-  
-  /* --- MAIN OPTIONS SCREEN LOOP: --- */
-
-  blinking = 0;
-  opt = 0;
-  done = 0;
-  quit = 0;
-  
-  do
-    {
-      last_time = SDL_GetTicks();
-      old_opt = opt;
-      
-      
-      /* Handle any incoming events: */
-      
-      while (SDL_PollEvent(&event) > 0)
-	{
-	  if (event.type == SDL_QUIT)
-	    {
-	      /* Window close event - quit! */
-	      
-	      quit = 1;
-	      done = 1;
-	    }
-
-	  else if (event.type == SDL_MOUSEBUTTONDOWN)
-            {
-              /* FIXME Options screen should have "OK" button to accept choices with mouse DSB */
-              /* figure out opt based on mouse coordinates */
-              if (/* event.button.x >= left && */ /* don't see any reason to filter on x values here*/
-	          /* event.button.x <= left + width && */  
-		  event.button.y >= images[IMG_OPTIONS]->h + 8 &&
-		  event.button.y <= (images[IMG_OPTIONS]->h + 8 +
-		                   NUM_OPTS * images[IMG_TUX_HELMET1]->h))
-	      {
-	        opt = ((event.button.y - (images[IMG_OPTIONS]->h + 8)) /
-	               images[IMG_TUX_HELMET1]->h);
-                update_selected_option(opt);
-	      }
-              /* handle ranges below list as last option */
-              if (event.button.y > (images[IMG_OPTIONS]->h + 8 +
-	      	                   NUM_OPTS * images[IMG_TUX_HELMET1]->h)
-                  &&
-
-                  event.button.y <= (images[IMG_OPTIONS]->h + 8 +
-	      	                    NUM_OPTS * images[IMG_TUX_HELMET1]->h)
-                                    + (images[IMG_OPT_RNG_1_5]->h))                          
-              { 
-                opt = OPT_Q_RANGE;
-                update_selected_option(opt);
-              }
-            }
-
-	  else if (event.type == SDL_KEYDOWN)
-	    {
-	      key = event.key.keysym.sym;
-	      
-	      if (key == SDLK_ESCAPE)
-		{
-                  /* Don't leave options screen if all operations deselected! - DSB */
-                  /* fixes Debian bug #336272 */
-                  int i;
-                  int at_least_one_oper = 0;
-                  for (i =0; i < NUM_OPERS; i++)
-                  {
-                    if (opers[i])
-                      at_least_one_oper = 1;
-                  }
-                  if (at_least_one_oper)
-                    done = 1;
-		}
-	      else if (key == SDLK_DOWN)
-		{
-		  if (opt < NUM_OPTS - 1)
-		  {
-		    opt++;
-
-		    playsound(SND_POP);
-		  }
-		}
-	      else if (key == SDLK_UP)
-		{
-		  if (opt > 0)
-		  {
-		    opt--;
-		    playsound(SND_POP);
-		  }
-		}
-	      else if (key == SDLK_RETURN ||
-		       key == SDLK_KP_ENTER ||
-		       key == SDLK_SPACE)
-              {
-              /* code moved into function taking opt as argument so the same code can be   */
-              /* used to handle mouse events DSB */
-                update_selected_option(opt);
-              }
-	    }
-	}
-      
-      
-      /* Erase Tux (cursor) */
-      
-      if (opt != old_opt)
-	{
-	  blinking = 0;
-	  
-	  dest.x = 32;
-	  dest.y = (images[IMG_OPTIONS]->h + 8 + 
-		    (old_opt * images[IMG_TUX_HELMET1]->h));
-	  dest.w = images[IMG_TUX_HELMET1]->w;
-	  dest.h = images[IMG_TUX_HELMET1]->h;
-	  
-	  SDL_FillRect(screen, &dest,
-		       SDL_MapRGB(screen->format, 0, 0, 0));
-	}
-      
-      
-      /* Handling Tux (cursor) blinking: */
-      
-      if ((rand() % 50) == 0 && blinking == 0)
-	{
-	  blinking = 6;
-	}
-      
-      if (blinking > 0)
-	blinking--;
-      
-      
-      /* Draw Tux (cursor) */
-      
-      dest.x = 32;
-      dest.y = images[IMG_OPTIONS]->h + 8 + (opt * images[IMG_TUX_HELMET1]->h);
-      dest.w = images[IMG_TUX_HELMET1]->w;
-      dest.h = images[IMG_TUX_HELMET1]->h;
-      
-      img = IMG_TUX_HELMET1;
-      
-      if (blinking >= 4 || (blinking >= 1 && blinking < 2))
-	img = IMG_TUX_HELMET2;
-      else if (blinking >= 2 && blinking < 4)
-	img = IMG_TUX_HELMET3;
-     
-      SDL_BlitSurface(images[img], NULL, screen, &dest);
-      
-      SDL_Flip(screen);
-
-      
-      /* Pause (keep frame-rate event) */
-      
-      now_time = SDL_GetTicks();
-      if (now_time < last_time + (1000 / 20))
-	{
-	  SDL_Delay(last_time + (1000 / 20) - now_time);
-	}
-    }
-  while (!done);
-
-
-  /* Return the chosen command: */
-  
-  return quit;
-}
 
 /********************************************************************/
 /*  "Public Methods" for options struct:                            */
@@ -336,11 +59,18 @@ int Opts_Initialize(void)
   /* set general game options */
   game_options->per_user_config = DEFAULT_PER_USER_CONFIG;
   game_options->use_sound = DEFAULT_USE_SOUND;
+  game_options->menu_sound = DEFAULT_MENU_SOUND;
+  game_options->menu_music = DEFAULT_MENU_MUSIC;
   game_options->fullscreen = DEFAULT_FULLSCREEN;
   game_options->use_bkgd = DEFAULT_USE_BKGD;
+  game_options->help_mode = DEFAULT_HELP_MODE;
   game_options->demo_mode = DEFAULT_DEMO_MODE;
   game_options->oper_override = DEFAULT_OPER_OVERRIDE;
   game_options->use_keypad = DEFAULT_USE_KEYPAD;
+  game_options->use_igloos = DEFAULT_USE_IGLOOS;
+  game_options->allow_pause = DEFAULT_ALLOW_PAUSE;
+  game_options->bonus_comet_interval = DEFAULT_BONUS_COMET_INTERVAL;
+  game_options->bonus_speed_ratio = DEFAULT_BONUS_SPEED_RATIO;
   game_options->speed = DEFAULT_SPEED;
   game_options->allow_speedup = DEFAULT_ALLOW_SPEEDUP;
   game_options->speedup_factor = DEFAULT_SPEEDUP_FACTOR;
@@ -356,6 +86,7 @@ int Opts_Initialize(void)
   game_options->danger_level_speedup = DEFAULT_DANGER_LEVEL_SPEEDUP;
   game_options->danger_level_max = DEFAULT_DANGER_LEVEL_MAX;
   game_options->city_expl_handicap = DEFAULT_CITY_EXPL_HANDICAP;
+  game_options->last_score = DEFAULT_LAST_SCORE;
 
   game_options->num_cities = DEFAULT_NUM_CITIES;   /* MUST BE AN EVEN NUMBER! */
   game_options->num_bkgds = DEFAULT_NUM_BKGDS;
@@ -388,10 +119,26 @@ void Opts_SetPerUserConfig(int val)
 
 void Opts_SetUseSound(int val)
 {
-  game_options->use_sound = int_to_bool(val);
+  if (val == -1)
+    game_options->use_sound = val;
+  else if (game_options->use_sound != -1)
+    game_options->use_sound = int_to_bool(val);
 }
 
 
+void Opts_SetMenuSound(int val)
+{
+  game_options->menu_sound = int_to_bool(val);
+}
+
+
+void Opts_SetMenuMusic(int val)
+{
+  game_options->menu_music = int_to_bool(val);
+}
+
+
+/* FIXME need to actually change screen resolution when this is called */
 void Opts_SetFullscreen(int val)
 {
   game_options->fullscreen = int_to_bool(val);
@@ -404,6 +151,12 @@ void Opts_SetUseBkgd(int val)
 }
 
 
+void Opts_SetHelpMode(int val)
+{
+  game_options->help_mode = int_to_bool(val);
+}
+
+
 void Opts_SetDemoMode(int val)
 {
   game_options->demo_mode = int_to_bool(val);
@@ -422,6 +175,42 @@ void Opts_SetUseKeypad(int val)
 }
 
 
+void Opts_SetAllowPause(int val)
+{
+  game_options->allow_pause = int_to_bool(val);
+}
+
+
+void Opts_SetUseIgloos(int val)
+{
+  game_options->use_igloos = int_to_bool(val);
+}
+
+
+void Opts_SetBonusCometInterval(int val)
+{
+  if (val < 0)
+    val = 0;
+  game_options->bonus_comet_interval = val;
+}
+
+
+void Opts_SetBonusSpeedRatio(float val)
+{
+  if (val < 1)
+  {
+    val = 1;
+    fprintf(stderr,"bonus_speed_ratio must be at least 1, resetting accordingly.\n");
+  }
+  if (val > MAX_BONUS_SPEED_RATIO)
+  {
+    val = MAX_BONUS_SPEED_RATIO;
+    fprintf(stderr,"Warning: requested bonus_speed_ratio above maximum, setting to %g.\n",MAX_BONUS_SPEED_RATIO);
+  }
+  game_options->bonus_speed_ratio = val;
+}
+
+
 void Opts_SetSpeed(float val)
 {
   if (val < MINIMUM_SPEED)
@@ -650,7 +439,11 @@ void Opts_SetSoundHWAvailable(int val)
   game_options->sound_hw_available = int_to_bool(val);
 }
 
-
+/* Allows game() to store score for high score table code: */
+void Opts_SetLastScore(int val)
+{
+  game_options->last_score = val;
+}
 
 /* "Get" functions for tuxmath options struct: */
 int Opts_PerUserConfig(void)
@@ -671,8 +464,29 @@ int Opts_UseSound(void)
     fprintf(stderr, "\nOpts_UseSound(): game_options not valid!\n");
     return GAME_OPTS_INVALID;
   }
-  return game_options->use_sound;
+  return game_options->use_sound > 0;
+}
 
+
+int Opts_MenuSound(void)
+{
+  if (!game_options)
+  {
+    fprintf(stderr, "\nOpts_MenuSound(): game_options not valid!\n");
+    return GAME_OPTS_INVALID;
+  }
+  return game_options->menu_sound;
+}
+
+
+int Opts_MenuMusic(void)
+{
+  if (!game_options)
+  {
+    fprintf(stderr, "\nOpts_MenuMusic(): game_options not valid!\n");
+    return GAME_OPTS_INVALID;
+  }
+  return game_options->menu_music;
 }
 
 
@@ -698,6 +512,17 @@ int Opts_UseBkgd(void)
 }
 
 
+int Opts_HelpMode(void)
+{
+  if (!game_options)
+  {
+    fprintf(stderr, "\nOpts_HelpMode(): game_options not valid!\n");
+    return GAME_OPTS_INVALID;
+  }
+  return game_options->help_mode;
+}
+
+
 int Opts_DemoMode(void)
 {
   if (!game_options)
@@ -731,6 +556,49 @@ int Opts_UseKeypad(void)
 }
 
 
+int Opts_AllowPause(void)
+{
+  if (!game_options)
+  {
+    fprintf(stderr, "\nOpts_AllowPause(): game_options not valid!\n");
+    return GAME_OPTS_INVALID;
+  }
+  return game_options->allow_pause;
+}
+
+
+int Opts_UseIgloos(void)
+{
+  if (!game_options)
+  {
+    fprintf(stderr, "\nOpts_UseIgloos(): game_options not valid!\n");
+    return GAME_OPTS_INVALID;
+  }
+  return game_options->use_igloos;
+}
+
+int Opts_BonusCometInterval(void)
+{
+  if (!game_options)
+  {
+    fprintf(stderr, "\nOpts_BonusCometInterval(): game_options not valid!\n");
+    return GAME_OPTS_INVALID;
+  }
+  return game_options->bonus_comet_interval;
+}
+
+
+float Opts_BonusSpeedRatio(void)
+{
+  if (!game_options)
+  {
+    fprintf(stderr, "\nOpts_BonusSpeedRatio(): game_options not valid!\n");
+    return GAME_OPTS_INVALID;
+  }
+  return game_options->bonus_speed_ratio;
+}
+
+
 float Opts_Speed(void)
 {
   if (!game_options)
@@ -818,7 +686,7 @@ int Opts_MaxComets(void)
   return game_options->max_comets;
 }
 
-
+/* FIXME maybe not good idea to have a malloc() in a function like this: */
 char* Opts_NextMission(void)
 {
   char* str;
@@ -841,6 +709,17 @@ int Opts_SaveSummary(void)
 }
 
 
+int Opts_LastScore(void)
+{
+  if (!game_options)
+  {
+    fprintf(stderr, "\nOpts_LastScore(): game_options not valid!\n");
+    return GAME_OPTS_INVALID;
+  }
+  return game_options->last_score;
+}
+
+
 int Opts_UseFeedback(void)
 {
   if (!game_options)
@@ -919,7 +798,7 @@ int Opts_UsingSound(void)
     fprintf(stderr, "\nOpts_UsingSound(): game_options not valid!\n");
     return GAME_OPTS_INVALID;
   }
-  return (game_options->use_sound && game_options->sound_hw_available);
+  return (game_options->use_sound>0 && game_options->sound_hw_available);
 }
 
 /********************************************************************/
@@ -927,170 +806,7 @@ int Opts_UsingSound(void)
 /********************************************************************/
 
 
-/* can be called by either keystroke or mouse click - moved into */
-/* separate function to reduce code duplication - DSB */
-static void update_selected_option(int option)
-{
-  {
-    if (option >= OPT_OP_ADD && option < OPT_OP_ADD + NUM_OPERS)
-    {
-      /* toggle selection of math operation - old opers array */
-      /* FIXME opers[] to go away                             */
-      opers[option - OPT_OP_ADD] = !opers[option - OPT_OP_ADD];
-      /* toggle selection in new MathCards backend: */
-      switch (option)
-      {
-        case OPT_OP_ADD:
-        {
-          MC_SetAddAllowed(!MC_AddAllowed());
-          break;
-	}
-        case OPT_OP_SUB:
-        {
-          MC_SetSubAllowed(!MC_SubAllowed());
-          break;
-        }
-        case OPT_OP_MUL:
-        {
-          MC_SetMultAllowed(!MC_MultAllowed());
-          break;
-        }
-        case OPT_OP_DIV:
-        {
-          MC_SetDivAllowed(!MC_DivAllowed());
-          break;
-        }
-      }
- 
-      dest.x = screen->w - images[IMG_OPT_CHECK]->w - 16;
-      dest.y = (images[IMG_OPTIONS]->h + 8 +
-               ((option - OPT_OP_ADD) *
-               images[IMG_TUX_HELMET1]->h));
 
-      SDL_BlitSurface(images[IMG_OPT_CHECK + opers[option - OPT_OP_ADD]],
-                      NULL,
-                      screen,
-                      &dest);
-    }
-
-    else if (option == OPT_A_MAX)
-    {
-      MC_SetMaxAnswer((MC_MaxAnswer() * 2) / 3);
-      if (MC_MaxAnswer() < 12)
-        MC_SetMaxAnswer(144);
-
-      dest.x = screen->w - ((images[IMG_NUMS]->w / 14) * 4) - 16;
-      dest.y = (images[IMG_OPTIONS]->h + 8 +
-               ((option - OPT_OP_ADD) *
-               images[IMG_TUX_HELMET1]->h));
-      dest.w = ((images[IMG_NUMS]->w / 14) * 4);
-      dest.h = images[IMG_OPT_MAX_ANSWER]->h;
-
-      SDL_FillRect(screen, &dest, SDL_MapRGB(screen->format, 0, 0, 0));
-      snprintf(tmp_str, sizeof(tmp_str), "%04d", MC_MaxAnswer());
-      draw_nums(tmp_str,
-                screen->w - ((images[IMG_NUMS]->w / 14) * 2) - 16,
-                (images[IMG_OPTIONS]->h + 8 +
-                ((option - OPT_OP_ADD) *
-                images[IMG_TUX_HELMET1]->h)) +
-                images[IMG_OPT_MAX_ANSWER]->h);
-    }
-
-    else if (option == OPT_A_SPEED)
-    {
-      game_options->speed = game_options->speed - 0.1;
-      if (game_options->speed < 0.1)
-        game_options->speed = 5;
-
-      dest.x = screen->w - ((images[IMG_NUMS]->w / 14) * 4) - 16;
-      dest.y = (images[IMG_OPTIONS]->h + 8 +
-               ((option - OPT_OP_ADD) *
-               images[IMG_TUX_HELMET1]->h));
-      dest.w = ((images[IMG_NUMS]->w / 14) * 4);
-      dest.h = images[IMG_OPT_SPEED]->h;
-
-      SDL_FillRect(screen, &dest, SDL_MapRGB(screen->format, 0, 0, 0));
-
-      snprintf(tmp_str, sizeof(tmp_str), "%.1f", game_options->speed);
-      draw_nums(tmp_str,
-                screen->w - ((images[IMG_NUMS]->w / 14) * 2) - 16,
-                (images[IMG_OPTIONS]->h + 8 +
-                ((option - OPT_OP_ADD) *
-                images[IMG_TUX_HELMET1]->h)) +
-                images[IMG_OPT_SPEED]->h);
-    }
-
-    else if (option == OPT_Q_RANGE)
-    {
-    /* FIXME question ranges now handled in MathCards */
-    /* Change which ranges are available: */
-      range_bits = range_bits + 1;
-      if (range_bits >= (1 << NUM_Q_RANGES))
-        range_bits = 1;
-
-      for (j = 0; j < NUM_Q_RANGES; j++)
-      {
-        if ((range_bits & (1 << j)) != 0)
-          range_enabled[j] = 1;
-        else
-          range_enabled[j] = 0;
-      }
-
-      /* Redraw ranges: */
-      x = 32 + images[IMG_TUX_HELMET1]->w + 4 + 64;
-      y = (images[IMG_OPTIONS]->h + 8 +
-          (option * images[IMG_TUX_HELMET1]->h)) +
-           images[IMG_TUX_HELMET1]->h;
-
-      for (j = 0; j < NUM_Q_RANGES; j++)
-      {
-        dest.x = x;
-        dest.y = y;
-
-        SDL_BlitSurface(images[IMG_OPT_RNG_1_5 + j * 2 +
-                        range_enabled[j]],
-                        NULL,
-                        screen, 
-                        &dest);
-
-        x = x
-            + images[IMG_OPT_RNG_1_5 + j * 2 + range_enabled[j]]-> w 
-            + 16;
-      }
-      
-      /* update settings in MathCards: */
-      {
-        int lowest_range, highest_range, minimum, maximum;
-        lowest_range = NUM_Q_RANGES - 1;
-        highest_range = 0;
-
-        /* find lowest and highest enabled ranges */
-        for (j = 0; j < NUM_Q_RANGES; j++)
-        {
-          if (range_enabled[j] && j < lowest_range)
-            lowest_range = j;
-          if (range_enabled[j] && j > highest_range)
-            highest_range = j;
-        } 
-        minimum = ranges[lowest_range].min;
-        maximum = ranges[highest_range].max;
-
-        /* update MathCards: */
-        MC_SetAddMin(minimum);
-        MC_SetAddMax(maximum);
-        MC_SetSubMin(minimum);
-        MC_SetSubMax(maximum);
-        MC_SetMultMin(minimum);
-        MC_SetMultMax(maximum);
-        MC_SetDivMin(minimum);
-        MC_SetDivMax(maximum);
-      }
-    }
- 
-    /* same sound for all option updates */
-    playsound(SND_LASER);
-  }
-}
 
 
 /* to prevent option settings in math_opts from getting set to */
@@ -1147,7 +863,7 @@ void print_game_options(FILE* fp, int verbose)
   {
     fprintf (fp, "\n# Self-explanatory, default is 1:\n");
   }
-  fprintf(fp, "use_sound = %d\n", game_options->use_sound);
+  fprintf(fp, "use_sound = %d\n", game_options->use_sound>0);
 
   if(verbose)
   {
