@@ -14,12 +14,12 @@
   dbruce@tampabay.rr.com
 
   Part of "Tux4Kids" Project
-  http://www.tux4kids.net/
+  http://tux4kids.alioth.debian.org/
   Subversion repository:
-  https://svn.tux4kids.net/tuxmath
+  https://svn.debian.alioth.org/tux4kids/tuxmath/
 
  
-  August 26, 2001 - August 8, 2006.
+  August 26, 2001 - Sept 18, 2007.
 */
 
 
@@ -31,6 +31,7 @@
 //#include <unistd.h>
 
 #include <SDL.h>
+#include <SDL_ttf.h>
 
 #ifndef NOSOUND
 #include <SDL_mixer.h>
@@ -44,13 +45,29 @@
 #include "setup.h"
 #include "fileops.h"
 #include "game.h"
-
-
+#include "titlescreen.h"
+#include "highscore.h"
+#include "SDL_extras.h"
 
 /* Global data used in setup.c:              */
 /* (These are now 'extern'd in "tuxmath.h") */
 SDL_Surface* screen;
 SDL_Surface* images[NUM_IMAGES];
+/* Need special handling to generate flipped versions of images. This
+   is a slightly ugly hack arising from the use of the enum trick for
+   NUM_IMAGES. */
+#define NUM_FLIPPED_IMAGES 6
+SDL_Surface* flipped_images[NUM_FLIPPED_IMAGES];
+int flipped_img_lookup[NUM_IMAGES];
+
+const int flipped_img[] = {
+  IMG_PENGUIN_WALK_ON1,
+  IMG_PENGUIN_WALK_ON2,
+  IMG_PENGUIN_WALK_ON3,
+  IMG_PENGUIN_WALK_OFF1,
+  IMG_PENGUIN_WALK_OFF2,
+  IMG_PENGUIN_WALK_OFF3
+};
 
 #ifndef NOSOUND
 Mix_Chunk* sounds[NUM_SOUNDS];
@@ -63,8 +80,9 @@ void initialize_options(void);
 void handle_command_args(int argc, char* argv[]);
 void initialize_SDL(void);
 void load_data_files(void);
+void generate_flipped_images(void);
 
-int initialize_game_options(void);
+//int initialize_game_options(void);
 void seticon(void);
 void usage(int err, char * cmd);
 
@@ -84,6 +102,8 @@ void setup(int argc, char * argv[])
   initialize_SDL();
   /* Read image and sound files: */
   load_data_files();
+ /* Generate flipped versions of walking images */
+  generate_flipped_images();
 }
 
 
@@ -106,7 +126,6 @@ void initialize_options(void)
   /* initialize game_options struct with defaults DSB */
   if (!Opts_Initialize())
   {
-    printf("\nUnable to initialize game_options\n");
     fprintf(stderr, "\nUnable to initialize game_options\n");
     cleanup_on_error();
     exit(1);
@@ -131,8 +150,30 @@ void initialize_options(void)
       fprintf(stderr, "\nCould not find user's config file.\n");
       /* can still proceed using hard-coded defaults.         */
     }
+
+    /* If game being run for first time, try to write file: */
+    if (!write_user_config_file())
+    {
+      fprintf(stderr, "\nUnable to write user's config file.\n");
+    }
   }
 
+  /* Read the lessons directory to determine which lesson   */
+  /* files are available.                                   */
+  if (!parse_lesson_file_directory())
+    fprintf(stderr,"\nCould not parse the lesson file directory.\n");
+
+  /* Now set up high score tables: */
+  initialize_scores();
+  if (!read_high_scores())
+  {
+    fprintf(stderr, "\nCould not find high score table.\n");
+    /* (can still proceed).         */
+  }
+
+#ifdef TUXMATH_DEBUG
+  print_high_scores(stdout);  
+#endif
 }
 
 
@@ -141,7 +182,7 @@ void initialize_options(void)
 /* Handle any arguments passed from command line */
 void handle_command_args(int argc, char* argv[])
 {
-  int i, j, found;
+  int i;
 
   for (i = 1; i < argc; i++)
   {
@@ -186,16 +227,7 @@ void handle_command_args(int argc, char* argv[])
 	"--speed S        - set initial speed of the game\n"
 	"                   (S may be fractional, default is 1.0)\n"
         "--allownegatives - to allow answers to be less than zero\n"
-	"--operator OP    - to automatically play with particular operators\n"
-	"                   OP may be one of:\n");
-
-      for (j = 0; j < NUM_OPERS; j++)
-        printf("                   \"%s\"\n", oper_opts[j]);
-
-      printf("            or:\n");
-      
-      for (j = 0; j < NUM_OPERS; j++)
-        printf("                   \"%s\"\n", oper_alt_opts[j]);
+	);
 
       printf("\n");
     
@@ -258,7 +290,7 @@ void handle_command_args(int argc, char* argv[])
 	     strcmp(argv[i], "--quiet") == 0 ||
 	     strcmp(argv[i], "-q") == 0)
     {
-      Opts_SetUseSound(0);
+      Opts_SetUseSound(-1);  // prevent options files from overwriting
     }
     else if (strcmp(argv[i], "--version") == 0 ||
 	     strcmp(argv[i], "-v") == 0)
@@ -318,35 +350,6 @@ void handle_command_args(int argc, char* argv[])
       i++;
     }
 
-    /* FIXME this does not currently work */
-    else if (strcmp(argv[i], "--operator") == 0 ||
-	     strcmp(argv[i], "-o") == 0)
-    {
-      if (i >= argc - 1)
-      {
-	fprintf(stderr, "%s option requires an argument\n", argv[i]);
-	usage(1, argv[0]);
-      }
-     
-      found = 0; 
-      for (j = 0; j < NUM_OPERS; j++)
-      {
-	if (strcmp(argv[i + 1], oper_opts[j]) == 0 ||
-	    strcmp(argv[i + 1], oper_alt_opts[j]) == 0)
-	{
-          found = 1;
-          opers[j] = 1;
-	}
-      }
-
-      if (found == 0)
-      {
-	fprintf(stderr, "Unrecognized operator %s\n", argv[i + 1]);
-	usage(1, argv[0]);
-      }
-
-      i++;
-    }
     else
     /* TODO try to match unrecognized strings to config file names */
     {
@@ -370,6 +373,10 @@ void handle_command_args(int argc, char* argv[])
 
 void initialize_SDL(void)
 {
+  // Audio parameters
+  int frequency,channels,n_timesopened;
+  Uint16 format;
+
   /* Init SDL Video: */
   screen = NULL;
 
@@ -384,8 +391,22 @@ void initialize_SDL(void)
   }
 
 
+  if (TTF_Init() < 0)
+  {
+    fprintf( stderr, "Couldn't initialize SDL_ttf\n"
+           "The Simple DirectMedia error that occured was:\n"
+           "%s\n\n", SDL_GetError());
+    cleanup_on_error();
+    exit(2);
+  }
+
+  atexit(TTF_Quit); // Maybe this is redundant?
+
+
+
   #ifndef NOSOUND
   /* Init SDL Audio: */
+  Opts_SetSoundHWAvailable(0);  // By default no sound HW
   if (Opts_UseSound())
   { 
     if (SDL_Init(SDL_INIT_AUDIO) < 0)
@@ -394,62 +415,85 @@ void initialize_SDL(void)
             "\nWarning: I could not initialize audio!\n"
             "The Simple DirectMedia error that occured was:\n"
             "%s\n\n", SDL_GetError());
-      Opts_SetSoundHWAvailable(0);
     }
+    else {
+      //if (Mix_OpenAudio(44100, AUDIO_S16SYS, 2, 2048) < 0)
+      if (Mix_OpenAudio(22050, AUDIO_S16SYS, 2, 2048) < 0)
+      {
+	fprintf(stderr,
+		"\nWarning: I could not set up audio for 44100 Hz "
+		"16-bit stereo.\n"
+		"The Simple DirectMedia error that occured was:\n"
+		"%s\n\n", SDL_GetError());
+      }
+    }
+    n_timesopened = Mix_QuerySpec(&frequency,&format,&channels);
+    if (n_timesopened > 0)
+      Opts_SetSoundHWAvailable(1);
+    #ifdef TUXMATH_DEBUG
+    fprintf(stderr, "Sound mixer: frequency = %d, "
+                    "format = %x, "
+                    "channels = %d, "
+                    "n_timesopened = %d\n",
+                    frequency,format,channels,n_timesopened);
+    #endif
   }
-
- 
-  if (Opts_UseSound())
+  
+  #endif
   {
-    if (Mix_OpenAudio(44100, AUDIO_S16SYS, 2, 2048) < 0)
+    SDL_VideoInfo *videoInfo;
+    Uint32 surfaceMode;
+    videoInfo = SDL_GetVideoInfo();
+    if (videoInfo->hw_available)
     {
-      fprintf(stderr,
-	      "\nWarning: I could not set up audio for 44100 Hz "
-	      "16-bit stereo.\n"
+      surfaceMode = SDL_HWSURFACE;
+#ifdef TUXMATH_DEBUG    
+      printf("HW mode\n");
+#endif
+    }
+    else
+    {
+      surfaceMode = SDL_SWSURFACE;
+#ifdef TUXMATH_DEBUG    
+      printf("SW mode\n");
+#endif
+    }
+
+    if (Opts_Fullscreen())
+    {
+      screen = SDL_SetVideoMode(RES_X, RES_Y, PIXEL_BITS, SDL_FULLSCREEN | surfaceMode);
+      if (screen == NULL)
+      {
+        fprintf(stderr,
+              "\nWarning: I could not open the display in fullscreen mode.\n"
 	      "The Simple DirectMedia error that occured was:\n"
 	      "%s\n\n", SDL_GetError());
-      Opts_SetSoundHWAvailable(0);
+        Opts_SetFullscreen(0);
+      }
     }
-  }
-  #endif
 
-
-  if (Opts_Fullscreen())
-  {
-    screen = SDL_SetVideoMode(640, 480, 16, SDL_FULLSCREEN | SDL_HWSURFACE);
+    if (!Opts_Fullscreen())
+    {
+      screen = SDL_SetVideoMode(RES_X, RES_Y, PIXEL_BITS, surfaceMode);
+    }
 
     if (screen == NULL)
     {
       fprintf(stderr,
-              "\nWarning: I could not open the display in fullscreen mode.\n"
-	      "The Simple DirectMedia error that occured was:\n"
-	      "%s\n\n", SDL_GetError());
-      Opts_SetFullscreen(0);
-    }
-  }
-
-  if (!Opts_Fullscreen())
-  {
-    screen = SDL_SetVideoMode(640, 480, 16, SDL_HWSURFACE);
-  }
-
-  if (screen == NULL)
-  {
-    fprintf(stderr,
             "\nError: I could not open the display.\n"
 	    "The Simple DirectMedia error that occured was:\n"
 	    "%s\n\n", SDL_GetError());
-    cleanup_on_error();
-    exit(1);
+      cleanup_on_error();
+      exit(1);
+    }
+
+    seticon();
+
+    SDL_WM_SetCaption("Tux, of Math Command", "TuxMath");
   }
-
-  seticon();
-
-  SDL_WM_SetCaption("Tux, of Math Command", "TuxMath");
 }
 
 
-
 void load_data_files(void)
 {
   if (!load_image_data())
@@ -458,36 +502,49 @@ void load_data_files(void)
     cleanup_on_error();
     exit(1);
   }
-  
+
   if (!load_sound_data())
   {
     fprintf(stderr, "\nCould not load sound file - attempting to proceed without sound.\n");
     Opts_SetSoundHWAvailable(0);
   }
 
-  
-  /* FIXME what does this do? */
-//   for (i = images[IMG_LOADING]->h; i >= 0; i = i - 10)
-//     {
-//       SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 0, 0, 0));
-// 
-//       dest.x = (screen->w - images[IMG_TITLE]->w) / 2;
-//       dest.y = i;
-//       dest.w = images[IMG_TITLE]->w;
-//       dest.h = images[IMG_TITLE]->h;
-//       
-//       SDL_BlitSurface(images[IMG_TITLE], NULL, screen, &dest);
-//       SDL_Flip(screen);
-//       SDL_Delay(10);
-//     }
+   if (!load_default_font())
+  {
+    fprintf(stderr, "\nCould not load default font - exiting!\n");
+    cleanup_on_error();
+    exit(1);
+  }
 }
 
+
+
+/* Create flipped versions of certain images; also set up the flip
+   lookup table */
+void generate_flipped_images(void)
+{
+  int i;
+
+  /* Zero out the flip lookup table */
+  for (i = 0; i < NUM_IMAGES; i++)
+    flipped_img_lookup[i] = 0;
+
+  for (i = 0; i < NUM_FLIPPED_IMAGES; i++) {
+    flipped_images[i] = Flip(images[flipped_img[i]],1,0);
+    flipped_img_lookup[flipped_img[i]] = i;
+  }
+}
+
+
 /* save options and free heap */
 /* use for successful exit */
 void cleanup(void)
 {
-  write_user_config_file();
+  /* No longer write settings here, because we only */
+  /* want to save settings from certain types of games. */
+  //write_user_config_file();
   cleanup_memory();
+  exit(0);
 }
 
 
@@ -497,14 +554,66 @@ void cleanup(void)
 void cleanup_on_error(void)
 {
   cleanup_memory();
+  exit(1);
 }
 
 
 
 /* free any heap memory used during game DSB */
+/* and also quit SDL properly:               */
 void cleanup_memory(void)
 {
+  /* Free all images and sounds used by SDL: */
+  int i;
+  int frequency,channels,n_timesopened;
+  Uint16 format;
+
+  TTF_CloseFont(default_font);
+  TTF_Quit();
+
+  for (i = 0; i < NUM_IMAGES; i++)
+  {
+    if (images[i])
+      SDL_FreeSurface(images[i]);
+    images[i] = NULL;
+  }
+
+  for (i = 0; i < NUM_SOUNDS; i++)
+  {
+    if (sounds[i])
+      Mix_FreeChunk(sounds[i]);
+    sounds[i] = NULL;
+  }
+
+  for (i = 0; i < NUM_MUSICS; i++)
+  {
+    if (musics[i])
+      Mix_FreeMusic(musics[i]);
+    musics[i] = NULL;
+  }
+
+  for (i = 0; i < num_lessons; i++) {
+    free(lesson_list_titles[i]);
+    free(lesson_list_filenames[i]);
+  }
+  free(lesson_list_titles);
+  free(lesson_list_filenames);
+  free(lesson_list_goldstars);
+  lesson_list_titles = NULL;
+  lesson_list_filenames = NULL;
+  lesson_list_goldstars = NULL;
+
+  // Close the audio mixer. We have to do this at least as many times
+  // as it was opened.
+  n_timesopened = Mix_QuerySpec(&frequency,&format,&channels);
+  while (n_timesopened) {
+    Mix_CloseAudio();
+    n_timesopened--;
+  }
+
+  // Finally, quit SDL
   SDL_Quit();
+
   /* frees the game_options struct: */
   Opts_Cleanup();
   /* frees any heap used by MathCards: */
@@ -522,43 +631,38 @@ void cleanup_memory(void)
 void seticon(void)
 {
   int masklen;
-  Uint8 * mask;
-  SDL_Surface * icon;
+  Uint8* mask;
+  SDL_Surface* icon;
   
   
   /* Load icon into a surface: */
-  
-  icon = IMG_Load(DATA_PREFIX "/images/icon.png");
+  icon = IMG_Load(DATA_PREFIX "/images/icons/icon.png");
   if (icon == NULL)
-    {
-      fprintf(stderr,
-              "\nWarning: I could not load the icon image: %s\n"
-              "The Simple DirectMedia error that occured was:\n"
-              "%s\n\n", DATA_PREFIX "images/icon.png", SDL_GetError());
-      return;
-    }
+  {
+    fprintf(stderr,
+            "\nWarning: I could not load the icon image: %s\n"
+            "The Simple DirectMedia error that occured was:\n"
+            "%s\n\n", DATA_PREFIX "/images/icons/icon.png", SDL_GetError());
+    return;
+  }
   
   
   /* Create mask: */
-  
   masklen = (((icon -> w) + 7) / 8) * (icon -> h);
   mask = malloc(masklen * sizeof(Uint8));
   memset(mask, 0xFF, masklen);
   
   
   /* Set icon: */
-  
   SDL_WM_SetIcon(icon, mask);
   
   
   /* Free icon surface & mask: */
-  
   free(mask);
   SDL_FreeSurface(icon);
   
   
   /* Seed random-number generator: */
-  
   srand(SDL_GetTicks());
 }
 
@@ -577,7 +681,7 @@ void usage(int err, char * cmd)
    "          [--playthroughlist] [--answersfirst] [--answersmiddle]\n"
    "       %s [--fullscreen] [--nosound] [--nobackground]\n"
    "          [--demo] [--keypad] [--allownegatives]\n"
-   "          [--operator {add | subtract | multiply | divide} ...]\n"
+//   "          [--operator {add | subtract | multiply | divide} ...]\n"
    "          [--speed <val>]\n"
     "\n", cmd, cmd);
 
